/*
 * OCSP responder
 * by Massimiliano Pala (madwolf@openca.org)
 * OpenCA project 2000
 *
 * Copyright (c) 1999 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.OpenSSL.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    licensing@OpenSSL.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.OpenSSL.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

/* Static Data Definition */

static BIO *bio_err=NULL;
static LHASH *conf=NULL;
static char *section=NULL;

static int verbose = 0;

static char *ocsp_usage[]={
"\n",
"usage: ocsp <command> args\n",
"\n",
"Valid Commands are:\n",
"    request          - OCSP Request\n",
"    response         - OCSP Response\n",
"    csl              - Certificate Suspension List\n",
"    suspend   <ser>  - Suspend PEM formatted (-certs) Certificate(s)\n",
"    unsuspend <ser>  - UnSuspend PEM formatted (-certs) Certificate(s)\n",
"    status    <ser>  - Returns Certificate Status\n",
"    version          - Returns OCSP command version\n",
"\n",
"Valid args are:\n",
"    -in              - Input file\n",
"    -out             - Output file\n",
"    -new             - Create new request or response\n",
"    -inform          - Input format ( PEM, DER, TXT )\n",
"    -outform         - Output format ( PEM, DER,TXT )\n",
"    -config <file>   - Configuration File\n",
"    -verify          - Verify signature on a request or response\n",
"    -sign            - Sign a new request or response\n",
"    -cert <file>     - Requestor/Responder certificate to be used (PEM)\n",
"    -keyfile <file>  - Private OCSP key file (PEM)\n",
"    -key <pwd>       - Private OCSP key usage password\n",
"    -certs <file>    - PEM formatted certificate(s) list file\n",
"    -cacert <file>   - CA certificate\n",
"    -digest <md>     - Digest algorithm\n",
"    -verbose         - Talk alot while doing things\n",
"\n",
NULL
};

/* Default definition for index db file handling */
#define BASE_SECTION    	"ocsp"
#define CONFIG_FILE 		"ocsp.cnf"

#define ENV_DEFAULT_OCSP        "default_ocsp"

#define ENV_CERTS               "certs"
#define ENV_CA_DB               "CA_DB"
#define ENV_CA_CERTIFICATE	"ca_certificate"
#define ENV_OCSP_CERTIFICATE	"ocsp_certificate"
#define ENV_PRIVATE_KEY         "ocsp_key"
#define ENV_DEFAULT_MD          "default_md"

#define ENV_DATABASE            "database"

#define DB_type         	0
#define DB_exp_date     	1
#define DB_rev_date     	2
#define DB_serial       	3       /* index - unique */
#define DB_file         	4
#define DB_name         	5       /* index - unique for active */
#define DB_NUMBER       	6

#define DB_TYPE_UNK     	'U'
#define DB_TYPE_REV     	'R'
#define DB_TYPE_EXP     	'E'
#define DB_TYPE_VAL     	'V'
#define DB_TYPE_SUS     	'S'    /* suspended - new type */

/* Default definitions for OCSP opts */
#define	OCSP_NONE		0
#define OCSP_REQUEST_CMD	1
#define OCSP_RESPONSE_CMD	2
#define OCSP_CSL_CMD		3
#define OCSP_SUSPEND_CMD	4
#define OCSP_UNSUSPEND_CMD	5
#define OCSP_STATUS_CMD		6
#define OCSP_VERSION_CMD	7

/* Request/Response formats */
#define OCSP_ASN1_FORMAT	0
#define OCSP_TXT_FORMAT		1
#define OCSP_PEM_FORMAT		2

/* Inernal Table definitions */
#define OCSP_COMMANDS_SIZE	7
#define OCSP_FORMAT_SIZE	3

/* TABLES (COMMANDS/FORMATS) structures */
typedef struct tableDef {
	int 	type;
	char 	*name;
} TBL;

static TBL ocspCommandsTbl[] = {
	{ OCSP_REQUEST_CMD , "request" },
	{ OCSP_RESPONSE_CMD, "response" },
	{ OCSP_CSL_CMD,	     "csl" },
	{ OCSP_SUSPEND_CMD,  "suspend" },
	{ OCSP_UNSUSPEND_CMD,"unsuspend" },
	{ OCSP_STATUS_CMD,   "status" },
	{ OCSP_VERSION_CMD,   "version" }
}; 

static TBL ocspFormatsTbl[] = {
	{ OCSP_ASN1_FORMAT, 	"DER" },
	{ OCSP_TXT_FORMAT, 	"TXT" },
	{ OCSP_PEM_FORMAT , 	"PEM" }
}; 

typedef struct cert_file_st
        {
        char *name;
        int   status;
        int   reason;
        char *time;
        char *this;
        char *next;
        } CERT_FILE;

/* static variables definition */
static	char *key = NULL;

/* Special Define */
#define i2d_OCSP_REQ_bio(bp,o) \
	ASN1_i2d_bio(i2d_OCSP_REQ,bp, (unsigned char *)o )
 
/* Functions declaration */
static void lookup_fail(char *name,char *tag);
static unsigned long index_serial_hash(char **a);
static int index_serial_cmp(char **a, char **b);
static unsigned long index_name_hash(char **a);
static int index_name_qual(char **a);
static int index_name_cmp(char **a,char **b);
static int set_mode(X509 *x509, TXT_DB *db, char MODE);

OCSP_REQUEST *generateNewRequest( X509 *cert, X509 *cacert, 
			EVP_PKEY *pkey, EVP_MD *ocsp_digest,
			char *certsfile, int sign );

STACK *read_certs(char *file, int min);

int i2d_OCSP_REQ ( OCSP_REQUEST *a, unsigned char **pp);
int i2d_OCSP_REQ();

OCSP_CERTID *d2i_OCSP_CERTID();
int OCSP_SIGNATURE_free(OCSP_SIGNATURE *a);

static char get_status ( ASN1_INTEGER *serial, TXT_DB *db );
static int set_status(X509 *x509, TXT_DB *db, char MODE);

